capture log close
clear                            
set more off                 
set mem 100m      
set matsize 5000           
log using matrix4.log, replace

*Stata codes;
*Quasi-DM;
*2SLS estimation using mata which is much faster than using canned stata;
*brand-market dummy, exogenous prices, base share weighted discrete attributes;

use "/vol1/project/0213096/ny/ajae/normalize/spacedata.dta", clear



*number of observations;
scalar nobs = _N
display nobs

*define cross-section;
global cs "mpid"

*count # of cross-sections;
tab $cs,nofreq
scalar ncs=r(r)
display ncs

*list of distance metric variables for cross-price effects;
global dmlst "brand9 brand11 brand13 cat10 regdiet1 adcaff flavor35"

*list of continous metric variables for cross-price effects;
global cmlst ""

local numc: word count $cmlst

*cross-price not weighted by budget shares;
global dmplst ""

*cross-price weighted by base shares;
global ivdmplst ""

*cross-price weighted by continous attributes;
global cmplst ""

summarize pid
global nb=r(max)
display $nb
global plst ""
global xlst ""
global templst ""
global trenlst ""
global ivxlst ""

*generate brand-specific prices, group expenditures and temperature and their instruments;
forvalues j=1/$nb {
  g p`j'=log(p)*(pid==`j')
  g x`j'=(lnx-stonep)*(pid==`j')
  g temp`j'=temp*(pid==`j')
  g tren`j'=(qtr/1000)*(pid==`j')
  g ivx`j'=(ivlnx-laspep)*(pid==`j')
  global plst "$plst p`j'"
  global xlst "$xlst x`j'"
  global templst "$templst temp`j'"
  global trenlst "$trenlst tren`j'"
  global ivxlst "$ivxlst ivx`j'"
}

foreach v of global dmlst {
  g p_wa`v'=w0*p_a`v'
  replace p_h`v'=wh*p_h`v'
  replace p_b`v'=w0*p_b`v'
  global dmplst "$dmplst p_b`v'"
  global ivdmplst "$ivdmplst p_a`v'"
}

if (`numc'>0) {
foreach v of global cmlst {
  global cmplst "$cmplst p_c`v'"
 }
}


*full list of exogenous rhs variables;
global exolst "$dmplst $cmplst $plst $templst $trenlst"

*full list of endogenous rhs variables;
global endolst "$xlst"

*full list of instrumental variables;
global ivlst "$ivxlst $ivdmplst"


*manually de-mean budget shares;
egen me_w=mean(w),by($cs)
egen av_w=mean(w)
g md_w = w-me_w+av_w


local nexo: word count $exolst
local nendo: word count $endolst

*group-mean endogenous variables;
global me_endolst ""

*mean-deviated endogenous variables;
global md_endolst ""

foreach v of global endolst {
        egen me_`v'=mean(`v'), by($cs)
        egen av_`v'=mean(`v')
        g md_`v'=`v'-me_`v'+av_`v'
        global me_endolst "$me_endolst me_`v'"
        global md_endolst "$md_endolst md_`v'"    
}

*gnerate mean-deviated values for exogenous variables;
global md_exolst ""
global me_exolst ""

if (`nexo'>0) {
   foreach v of global exolst {
     egen me_`v'=mean(`v'), by($cs)
     egen av_`v'=mean(`v')
     g md_`v'=`v'-me_`v'+av_`v'
     global me_exolst "$me_exolst me_`v'"
     global md_exolst "$md_exolst md_`v'"
   }
}

*gnerate mean-deviated values for instrumental variables;
global md_ivlst ""
global me_ivlst ""

local niv: word count $ivlst

if (`niv'>0) {
   foreach v of global ivlst {
     egen me_`v'=mean(`v'), by($cs)
     egen av_`v'=mean(`v')
     g md_`v'=`v'-me_`v'+av_`v'
     global me_ivlst "$me_ivlst me_`v'"
     global md_ivlst "$md_ivlst md_`v'"
   }
}


mata

t = st_numscalar("nobs")
ncs = st_numscalar("ncs")

mpid_qtr = st_data(.,"mpid qtr")

/*instruments*/
md_z = st_data(.,"$md_ivlst $md_exolst uno")
/*regressors*/
md_x1 = st_data(.,"$md_endolst")
md_x2 = st_data(.,"$md_exolst uno")
/*dependent variable*/
md_y = st_data(.,"md_w")

/*group-mean instruments*/
me_z = st_data(.,"$me_ivlst $me_exolst uno")
/*group-mean regressors*/
me_x1 = st_data(.,"$me_endolst")
me_x2 = st_data(.,"$me_exolst uno")
/*group-mean dependent variable*/
me_y = st_data(.,"me_w")

/*instruments in level*/
z = st_data(.,"$ivlst $exolst uno")
/*regressors in level*/
x1 = st_data(.,"$endolst")
x2 = st_data(.,"$exolst uno")
/*dependent variable in level*/
y = st_data(.,"w")

/*number of instruments*/
q = rows(md_z')
/*number of parameters*/
p = rows((md_x1,md_x2)')

/*sample mean dependent variable*/
av_y = st_data(.,"av_w")

/*save intermediate matrices to be deleted when complete*/
mata matsave int_m4 t ncs mpid_qtr md_z md_x1 md_x2 md_y me_z me_x1 me_x2 me_y x1 x2 z y q p av_y, replace

end

clear all


mata

mata matuse int_m4, replace

mzz = cross(md_z,md_z)/t
invmzz =invsym(cross(md_z,md_z)/t)


/*2sls coefficient vector*/
theta = invsym(cross((md_x1,md_x2),md_z)*(invmzz/t)*cross(md_z,(md_x1,md_x2)))*            
        cross((md_x1,md_x2),md_z)*(invmzz/t)*cross(md_z,md_y) 

/*group fixed effect estimates*/
u = me_y - (me_x1,me_x2)*theta
/*residuals*/
e = y - (x1,x2)*theta - u
s_ee = cross(e,e)
s_yy = cross((md_y-av_y),(md_y-av_y))

/*gt on p8 of Hall and Pelletier*/
gt = cross(md_z,e)/t

mata drop md_y me_y me_x2 x1 av_y 

/*parameter var-cov matrix*/
cov = (s_ee/(t-p-ncs+1))*invsym((cross((md_x1,md_x2),md_z)/t)*invmzz
      *(cross(md_z,(md_x1,md_x2))/t))/t


/*group fixed effect estimates for endogenous rhs variables*/
x1_u = me_x1 - me_z*((invmzz/t)*cross(md_z,md_x1))
/*predicted endogenous rhs variables in level*/
x1_h = z*((invmzz/t)*cross(md_z,md_x1)) + x1_u
/*predicted budget share in levels based on predicted endogenous variables*/
y_h = (x1_h,x2)*theta + u
/*residuals based on predicted rhs endogenous variables*/
e_h = y - y_h
s_gee = cross(e_h,e_h)


/*present coefficient estimates and standard errors*/
est = theta,sqrt(diagonal(cov))

/*export mata coefficient and covariance matrices to stata matrices*/
st_matrix("b",theta')
st_matrix("cov",cov')

/*save vectors to stata data*/
st_addobs(rows(y_h))
st_addvar("double",("mpid","qtr","w_h"))
st_store(.,("mpid","qtr","w_h"),(mpid_qtr,y_h))

for (i=1;i<=cols(x1_h);i++) {
   xname = "x1_h"+strofreal(i)
   st_addvar("double",xname)
   st_store(.,xname,x1_h[.,i])
}

mata drop md_x1 md_x2 me_x1 me_z x1_h y y_h x1_u x1_h x2 e_h z y u cov


/*traditional r squared*/
r2 = 1-s_ee/s_yy
/*generalized r squared*/
gr2 = 1-s_gee/s_yy
/*Pesaran and Smith p708*/
epe_syy = (gt)'*(invmzz)*(gt*t)/s_yy
/*Sargan's overidentification test*/
/*under classical assumption 2.5 about residual independence in Hall (2005)*/
sargan_m4 = (gt)'*invmzz*(gt*t)/(s_ee/t)
/*under eq 2.28 in Hall (2005)*/
sargan_alt_m4 = (gt)'*invsym(cross(e:*md_z,e:*md_z)/t)*(gt*t)
sargan_df = q-p


r2
gr2
epe_syy
sargan_m4
sargan_alt_m4
sargan_df
est



end

describe

/*save parts_4, replace*/

erase "/vol1/project/0213096/ny/ajae/normalize/int_m4.mmat"

xsvmat b, saving(b4,replace)
use "/vol1/project/0213096/ny/ajae/normalize/b4.dta", clear
export sasxport b4, replace
erase "/vol1/project/0213096/ny/ajae/normalize/b4.dta"

xsvmat cov, saving(cov4,replace)
use "/vol1/project/0213096/ny/ajae/normalize/cov4.dta", clear
export sasxport cov4, replace
erase "/vol1/project/0213096/ny/ajae/normalize/cov4.dta"

log close

exit




/*The codes below are for conducting the Rivers and Vuong Test*/
/*They take a while to run if the data set is large*/

/*construct variables for Rivers and Vuong (2002) test*/

e_m4 = e
z_m4 = md_z
mzz_m4 = mzz

mata matsave xi_m4 e_m4 z_m4 mzz_m4 sargan_m4 sargan_alt_m4, replace

mata drop e_m4 e z_m4 md_z mzz_m4 mzz est

end

erase "/vol1/project/0213096/ny/ajae/normalize/int_m4.mmat"





log close

exit





